@@ -12,17 +12,17 @@
 /* map_sexp
  *
  */
-SEXP map_sexp(const SEXP sexp, SEXP (*const f)(SEXP));
+SEXP map_sexp(SEXP (*const f)(SEXP), const SEXP sexp);
 
 /* map_sexp_leafs
  *
  */
-SEXP map_sexp_leafs(const SEXP sexp, SEXP (*const f)(SEXP));
+SEXP map_sexp_leafs(SEXP (*const f)(SEXP), const SEXP sexp);
 
 /* map_sexp_inner_nodes
  *
  */
-SEXP map_sexp_inner_nodes(const SEXP sexp, SEXP (*const f)(SEXP));
+SEXP map_sexp_inner_nodes(SEXP (*const f)(SEXP), const SEXP sexp);
 
 // TODO
 
